"""
Preprocessor used in kaitai struct to encode data into Base64
encoded data
"""
import base64


class B64encode(object):  # pylint: disable=too-few-public-methods
    """
    b64decode preprocessor class
    """
    def __init__(self):
        pass

    def decode(self, in_bindata):  # pylint: disable=no-self-use
        """
        b64decode method
        """
        bindata = bytearray(in_bindata)
        return bytearray(base64.b64encode(bindata))



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import satnogsdecoders.process


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Opssat1(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field priority: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.header.priority
    :field source: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.header.source
    :field destination: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.header.destination
    :field destination_port: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.header.destination_port
    :field source_port: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.header.source_port
    :field reserved: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.header.reserved
    :field csp_hmac: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.header.hmac
    :field csp_xtea: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.header.xtea
    :field csp_rdp: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.header.rdp
    :field csp_crc: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.header.crc
    :field board_temperature: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.board_temperature
    :field pa_temperature: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.pa_temperature
    :field last_received_rssi: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.last_received_rssi
    :field last_received_rf_error: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.last_received_rf_error
    :field number_of_tx_packets_since_reboot: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.number_of_tx_packets_since_reboot
    :field number_of_rx_packets_since_reboot: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.number_of_rx_packets_since_reboot
    :field number_of_tx_bytes_since_reboot: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.number_of_tx_bytes_since_reboot
    :field number_of_rx_bytes_since_reboot: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.number_of_rx_bytes_since_reboot
    :field active_system_configuration: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.active_system_configuration
    :field reboot_number: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.reboot_number
    :field reboot_cause: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.reboot_cause
    :field last_valid_packet_timestamp: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.last_valid_packet_timestamp
    :field background_rssi_level: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.background_rssi_level
    :field tx_duty_time_since_reboot: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.tx_duty_time_since_reboot
    :field total_tx_packets: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.total_tx_packets
    :field total_rx_packets: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.total_rx_packets
    :field total_tx_bytes: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.total_tx_bytes
    :field total_rx_bytes: ax25_frame.payload.info.pld_id.payload_dscrm.opssat_payload.beacon_data.total_rx_bytes
    
    .. seealso::
       Source - https://github.com/esa/gr-opssat/blob/master/docs/os-uhf-specs.pdf
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Opssat1.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Opssat1.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 3:
                self.payload = Opssat1.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Opssat1.UiFrame(self._io, self, self._root)


    class CspBeaconData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.board_temperature = self._io.read_s2be()
            self.pa_temperature = self._io.read_s2be()
            self.last_received_rssi = self._io.read_s2be()
            self.last_received_rf_error = self._io.read_s2be()
            self.number_of_tx_packets_since_reboot = self._io.read_u4be()
            self.number_of_rx_packets_since_reboot = self._io.read_u4be()
            self.number_of_tx_bytes_since_reboot = self._io.read_u4be()
            self.number_of_rx_bytes_since_reboot = self._io.read_u4be()
            self.active_system_configuration = self._io.read_u1()
            self.reboot_number = self._io.read_u2be()
            self.reboot_cause = self._io.read_u4be()
            self.last_valid_packet_timestamp = self._io.read_u4be()
            self.background_rssi_level = self._io.read_s2be()
            self.tx_duty_time_since_reboot = self._io.read_u1()
            self.total_tx_packets = self._io.read_u4be()
            self.total_rx_packets = self._io.read_u4be()
            self.total_tx_bytes = self._io.read_u4be()
            self.total_rx_bytes = self._io.read_u4be()


    class CspHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self._root.id_magic
            if _on == 12625866:
                self.raw_csp_header = self._io.read_u4le()
            else:
                self.raw_csp_header = self._io.read_u4be()

        @property
        def source(self):
            if hasattr(self, '_m_source'):
                return self._m_source if hasattr(self, '_m_source') else None

            self._m_source = ((self.raw_csp_header >> 25) & 31)
            return self._m_source if hasattr(self, '_m_source') else None

        @property
        def source_port(self):
            if hasattr(self, '_m_source_port'):
                return self._m_source_port if hasattr(self, '_m_source_port') else None

            self._m_source_port = ((self.raw_csp_header >> 8) & 63)
            return self._m_source_port if hasattr(self, '_m_source_port') else None

        @property
        def destination_port(self):
            if hasattr(self, '_m_destination_port'):
                return self._m_destination_port if hasattr(self, '_m_destination_port') else None

            self._m_destination_port = ((self.raw_csp_header >> 14) & 63)
            return self._m_destination_port if hasattr(self, '_m_destination_port') else None

        @property
        def rdp(self):
            if hasattr(self, '_m_rdp'):
                return self._m_rdp if hasattr(self, '_m_rdp') else None

            self._m_rdp = ((self.raw_csp_header & 2) >> 1)
            return self._m_rdp if hasattr(self, '_m_rdp') else None

        @property
        def destination(self):
            if hasattr(self, '_m_destination'):
                return self._m_destination if hasattr(self, '_m_destination') else None

            self._m_destination = ((self.raw_csp_header >> 20) & 31)
            return self._m_destination if hasattr(self, '_m_destination') else None

        @property
        def priority(self):
            if hasattr(self, '_m_priority'):
                return self._m_priority if hasattr(self, '_m_priority') else None

            self._m_priority = (self.raw_csp_header >> 30)
            return self._m_priority if hasattr(self, '_m_priority') else None

        @property
        def reserved(self):
            if hasattr(self, '_m_reserved'):
                return self._m_reserved if hasattr(self, '_m_reserved') else None

            self._m_reserved = ((self.raw_csp_header >> 4) & 15)
            return self._m_reserved if hasattr(self, '_m_reserved') else None

        @property
        def xtea(self):
            if hasattr(self, '_m_xtea'):
                return self._m_xtea if hasattr(self, '_m_xtea') else None

            self._m_xtea = ((self.raw_csp_header & 4) >> 2)
            return self._m_xtea if hasattr(self, '_m_xtea') else None

        @property
        def hmac(self):
            if hasattr(self, '_m_hmac'):
                return self._m_hmac if hasattr(self, '_m_hmac') else None

            self._m_hmac = ((self.raw_csp_header & 8) >> 3)
            return self._m_hmac if hasattr(self, '_m_hmac') else None

        @property
        def crc(self):
            if hasattr(self, '_m_crc'):
                return self._m_crc if hasattr(self, '_m_crc') else None

            self._m_crc = (self.raw_csp_header & 1)
            return self._m_crc if hasattr(self, '_m_crc') else None


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Opssat1.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Opssat1.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Opssat1.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Opssat1.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Opssat1.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class OpssatPayloadPreprocessed(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw_payload_dscrm = self._io.read_bytes_full()
            _io__raw_payload_dscrm = KaitaiStream(BytesIO(self._raw_payload_dscrm))
            self.payload_dscrm = Opssat1.OpssatPayload(_io__raw_payload_dscrm, self, self._root)

        @property
        def payload_size(self):
            if hasattr(self, '_m_payload_size'):
                return self._m_payload_size if hasattr(self, '_m_payload_size') else None

            self._m_payload_size = self._io.size()
            return self._m_payload_size if hasattr(self, '_m_payload_size') else None


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            _on = self._parent.ax25_header.src_callsign_raw.callsign_ror.callsign
            if _on == u"DP0OPS":
                self.info = Opssat1.OpssatPayloadIdentify(self._io, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Opssat1.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Opssat1.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Opssat1.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class OpssatPayloadIdentify(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self._root.id_magic
            if _on == 904908480:
                self.pld_id = Opssat1.OpssatPayloadDscrm(self._io, self, self._root)
            elif _on == 12625866:
                self.pld_id = Opssat1.OpssatPayloadPreprocessed(self._io, self, self._root)
            elif _on == 3399991296:
                self.pld_id = Opssat1.OpssatPayloadPreprocessed(self._io, self, self._root)


    class CubeSatProtocol(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header = Opssat1.CspHeader(self._io, self, self._root)
            if  ((self.header.destination == 10) and (self.header.priority == 3) and (self.header.source == 5) and (self.header.destination_port == 31)) :
                self.beacon_data = Opssat1.CspBeaconData(self._io, self, self._root)



    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Opssat1.Callsign(_io__raw_callsign_ror, self, self._root)


    class OpssatPayload(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.opssat_payload = Opssat1.CubeSatProtocol(self._io, self, self._root)


    class OpssatPayloadDscrm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_payload_dscrm = self._io.read_bytes_full()
            _process = satnogsdecoders.process.Scrambler(169, 255, 7)
            self._raw_payload_dscrm = _process.decode(self._raw__raw_payload_dscrm)
            _io__raw_payload_dscrm = KaitaiStream(BytesIO(self._raw_payload_dscrm))
            self.payload_dscrm = Opssat1.OpssatPayload(_io__raw_payload_dscrm, self, self._root)

        @property
        def payload_size(self):
            if hasattr(self, '_m_payload_size'):
                return self._m_payload_size if hasattr(self, '_m_payload_size') else None

            self._m_payload_size = self._io.size()
            return self._m_payload_size if hasattr(self, '_m_payload_size') else None


    @property
    def id_magic(self):
        if hasattr(self, '_m_id_magic'):
            return self._m_id_magic if hasattr(self, '_m_id_magic') else None

        _pos = self._io.pos()
        self._io.seek(16)
        self._m_id_magic = self._io.read_u4be()
        self._io.seek(_pos)
        return self._m_id_magic if hasattr(self, '_m_id_magic') else None





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Sanosat1(KaitaiStruct):
    """:field callsign: sanosat1_telemetry.body.callsign
    :field packet_type: sanosat1_telemetry.body.packet_type
    :field com_temperature: sanosat1_telemetry.body.com_temperature
    :field battery_voltage: sanosat1_telemetry.body.battery_voltage
    :field charging_current: sanosat1_telemetry.body.charging_current
    :field battery_temperature: sanosat1_telemetry.body.battery_temperature
    :field radiation_level: sanosat1_telemetry.body.radiation_level
    :field no_of_resets: sanosat1_telemetry.body.no_of_resets
    :field antenna_deployment_status: sanosat1_telemetry.body.antenna_deployment_status
    :field beacon_type: sanosat1_telemetry.body.beacon_type
    :field digimessage: sanosat1_telemetry.body.decision.digimessage
    :field cwmessage: sanosat1_telemetry.body.decision.cwmessage
    :field com_temperature: sanosat1_telemetry.body.decision.com_temperature
    :field battery_temperature: sanosat1_telemetry.body.decision.battery_temperature
    :field charging_current: sanosat1_telemetry.body.decision.charging_current
    :field battery_voltage: sanosat1_telemetry.body.decision.battery_voltage
    :field antenna_deployment_status: sanosat1_telemetry.body.decision.antenna_deployment_status
    :field beacon_type: sanosat1_telemetry.body.decision.beacon_type
    :field callsign: sanosat1_telemetry.body.decision.callsign
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.sanosat1_telemetry = Sanosat1.Sanosat1TelemetryT(self._io, self, self._root)

    class WithDelimiter(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.delimiter = self._io.read_s4le()
            if not  ((self.delimiter == 65535)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.delimiter, self._io, u"/types/with_delimiter/seq/0")
            self.callsign = (KaitaiStream.bytes_terminate(self._io.read_bytes(7), 0, False)).decode(u"ASCII")
            if not  ((self.callsign == u"AM9NPQ")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/with_delimiter/seq/1")
            self.packet_type = self._io.read_s2le()
            self.com_temperature = self._io.read_s2le()
            self.battery_voltage = self._io.read_s2le()
            self.charging_current = self._io.read_s2le()
            self.battery_temperature = self._io.read_s2le()
            self.radiation_level = self._io.read_s2le()
            self.no_of_resets = self._io.read_s2le()
            self.antenna_deployment_status = self._io.read_u1()

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

            self._m_beacon_type = (u"GFSK" if 0 == 0 else u"GFSK")
            return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None


    class Rtty(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            if not  ((self.callsign == u"AM9NPQ") or (self.callsign == u"am9npq")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/rtty/seq/0")
            self.skip_dollar = self._io.read_s1()
            self.battery_temperature_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.charging_current_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.battery_voltage_before_dot_raw = (self._io.read_bytes_term(46, False, True, True)).decode(u"UTF-8")
            self.battery_voltage_after_dot_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.no_of_resets_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.antenna_deployment_status_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.radiation_level_raw = (self._io.read_bytes_term(63, False, True, True)).decode(u"UTF-8")
            self.checksum_rtty_raw = (self._io.read_bytes(2)).decode(u"UTF-8")
            if not  ((self.checksum_rtty_raw == self.checksum_compare)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.checksum_rtty_raw, self._io, u"/types/rtty/seq/9")

        @property
        def battery_temperature(self):
            if hasattr(self, '_m_battery_temperature'):
                return self._m_battery_temperature if hasattr(self, '_m_battery_temperature') else None

            self._m_battery_temperature = int(self.battery_temperature_raw)
            return self._m_battery_temperature if hasattr(self, '_m_battery_temperature') else None

        @property
        def checksum_battery_temperature_2(self):
            if hasattr(self, '_m_checksum_battery_temperature_2'):
                return self._m_checksum_battery_temperature_2 if hasattr(self, '_m_checksum_battery_temperature_2') else None

            self._m_checksum_battery_temperature_2 = (0 if (self.battery_temperature_raw)[2:3] == u"" else (int((self.battery_temperature_raw)[2:3], 16) + 48))
            return self._m_checksum_battery_temperature_2 if hasattr(self, '_m_checksum_battery_temperature_2') else None

        @property
        def battery_voltage_before_dot(self):
            if hasattr(self, '_m_battery_voltage_before_dot'):
                return self._m_battery_voltage_before_dot if hasattr(self, '_m_battery_voltage_before_dot') else None

            self._m_battery_voltage_before_dot = int(self.battery_voltage_before_dot_raw)
            return self._m_battery_voltage_before_dot if hasattr(self, '_m_battery_voltage_before_dot') else None

        @property
        def checksum_battery_voltage_before_dot(self):
            if hasattr(self, '_m_checksum_battery_voltage_before_dot'):
                return self._m_checksum_battery_voltage_before_dot if hasattr(self, '_m_checksum_battery_voltage_before_dot') else None

            self._m_checksum_battery_voltage_before_dot = (int(self.battery_voltage_before_dot_raw, 16) + 48)
            return self._m_checksum_battery_voltage_before_dot if hasattr(self, '_m_checksum_battery_voltage_before_dot') else None

        @property
        def battery_voltage_after_dot(self):
            if hasattr(self, '_m_battery_voltage_after_dot'):
                return self._m_battery_voltage_after_dot if hasattr(self, '_m_battery_voltage_after_dot') else None

            self._m_battery_voltage_after_dot = int(self.battery_voltage_after_dot_raw)
            return self._m_battery_voltage_after_dot if hasattr(self, '_m_battery_voltage_after_dot') else None

        @property
        def battery_voltage(self):
            if hasattr(self, '_m_battery_voltage'):
                return self._m_battery_voltage if hasattr(self, '_m_battery_voltage') else None

            self._m_battery_voltage = (self.battery_voltage_after_dot + (self.battery_voltage_before_dot * 100))
            return self._m_battery_voltage if hasattr(self, '_m_battery_voltage') else None

        @property
        def checksum_compare(self):
            if hasattr(self, '_m_checksum_compare'):
                return self._m_checksum_compare if hasattr(self, '_m_checksum_compare') else None

            self._m_checksum_compare = (self.checksum_rtty_raw if self.checksum_calculation == int(self.checksum_rtty_raw, 16) else u"checksum invalid")
            return self._m_checksum_compare if hasattr(self, '_m_checksum_compare') else None

        @property
        def radiation_level(self):
            if hasattr(self, '_m_radiation_level'):
                return self._m_radiation_level if hasattr(self, '_m_radiation_level') else None

            self._m_radiation_level = int(self.radiation_level_raw)
            return self._m_radiation_level if hasattr(self, '_m_radiation_level') else None

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

            self._m_beacon_type = (u"RTTY" if 0 == 0 else u"RTTY")
            return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

        @property
        def checksum_no_of_resets_raw_1(self):
            if hasattr(self, '_m_checksum_no_of_resets_raw_1'):
                return self._m_checksum_no_of_resets_raw_1 if hasattr(self, '_m_checksum_no_of_resets_raw_1') else None

            self._m_checksum_no_of_resets_raw_1 = (0 if (self.no_of_resets_raw)[1:2] == u"" else (int((self.no_of_resets_raw)[1:2], 16) + 48))
            return self._m_checksum_no_of_resets_raw_1 if hasattr(self, '_m_checksum_no_of_resets_raw_1') else None

        @property
        def checksum_charging_current_0(self):
            if hasattr(self, '_m_checksum_charging_current_0'):
                return self._m_checksum_charging_current_0 if hasattr(self, '_m_checksum_charging_current_0') else None

            self._m_checksum_charging_current_0 = (int((self.charging_current_raw)[0:1], 16) + 48)
            return self._m_checksum_charging_current_0 if hasattr(self, '_m_checksum_charging_current_0') else None

        @property
        def checksum_no_of_resets_raw_0(self):
            if hasattr(self, '_m_checksum_no_of_resets_raw_0'):
                return self._m_checksum_no_of_resets_raw_0 if hasattr(self, '_m_checksum_no_of_resets_raw_0') else None

            self._m_checksum_no_of_resets_raw_0 = (int((self.no_of_resets_raw)[0:1], 16) + 48)
            return self._m_checksum_no_of_resets_raw_0 if hasattr(self, '_m_checksum_no_of_resets_raw_0') else None

        @property
        def checksum_radiation_level_raw_1(self):
            if hasattr(self, '_m_checksum_radiation_level_raw_1'):
                return self._m_checksum_radiation_level_raw_1 if hasattr(self, '_m_checksum_radiation_level_raw_1') else None

            self._m_checksum_radiation_level_raw_1 = (0 if (self.radiation_level_raw)[1:2] == u"" else (int((self.radiation_level_raw)[1:2], 16) + 48))
            return self._m_checksum_radiation_level_raw_1 if hasattr(self, '_m_checksum_radiation_level_raw_1') else None

        @property
        def charging_current(self):
            if hasattr(self, '_m_charging_current'):
                return self._m_charging_current if hasattr(self, '_m_charging_current') else None

            self._m_charging_current = int(self.charging_current_raw)
            return self._m_charging_current if hasattr(self, '_m_charging_current') else None

        @property
        def checksum_battery_voltage_after_dot_1(self):
            if hasattr(self, '_m_checksum_battery_voltage_after_dot_1'):
                return self._m_checksum_battery_voltage_after_dot_1 if hasattr(self, '_m_checksum_battery_voltage_after_dot_1') else None

            self._m_checksum_battery_voltage_after_dot_1 = (int((self.battery_voltage_after_dot_raw)[1:2], 16) + 48)
            return self._m_checksum_battery_voltage_after_dot_1 if hasattr(self, '_m_checksum_battery_voltage_after_dot_1') else None

        @property
        def checksum_five_commata_and_one_dot(self):
            if hasattr(self, '_m_checksum_five_commata_and_one_dot'):
                return self._m_checksum_five_commata_and_one_dot if hasattr(self, '_m_checksum_five_commata_and_one_dot') else None

            self._m_checksum_five_commata_and_one_dot = 2
            return self._m_checksum_five_commata_and_one_dot if hasattr(self, '_m_checksum_five_commata_and_one_dot') else None

        @property
        def checksum_radiation_level_raw_0(self):
            if hasattr(self, '_m_checksum_radiation_level_raw_0'):
                return self._m_checksum_radiation_level_raw_0 if hasattr(self, '_m_checksum_radiation_level_raw_0') else None

            self._m_checksum_radiation_level_raw_0 = (int((self.radiation_level_raw)[0:1], 16) + 48)
            return self._m_checksum_radiation_level_raw_0 if hasattr(self, '_m_checksum_radiation_level_raw_0') else None

        @property
        def checksum_charging_current_2(self):
            if hasattr(self, '_m_checksum_charging_current_2'):
                return self._m_checksum_charging_current_2 if hasattr(self, '_m_checksum_charging_current_2') else None

            self._m_checksum_charging_current_2 = (0 if (self.charging_current_raw)[2:3] == u"" else (int((self.charging_current_raw)[2:3], 16) + 48))
            return self._m_checksum_charging_current_2 if hasattr(self, '_m_checksum_charging_current_2') else None

        @property
        def checksum_charging_current_1(self):
            if hasattr(self, '_m_checksum_charging_current_1'):
                return self._m_checksum_charging_current_1 if hasattr(self, '_m_checksum_charging_current_1') else None

            self._m_checksum_charging_current_1 = (0 if (self.charging_current_raw)[1:2] == u"" else (int((self.charging_current_raw)[1:2], 16) + 48))
            return self._m_checksum_charging_current_1 if hasattr(self, '_m_checksum_charging_current_1') else None

        @property
        def checksum_calculation(self):
            if hasattr(self, '_m_checksum_calculation'):
                return self._m_checksum_calculation if hasattr(self, '_m_checksum_calculation') else None

            self._m_checksum_calculation = (((((((((((((((((self.checksum_battery_temperature_0 ^ self.checksum_battery_temperature_1) ^ self.checksum_battery_temperature_2) ^ self.checksum_battery_temperature_3) ^ self.checksum_charging_current_0) ^ self.checksum_charging_current_1) ^ self.checksum_charging_current_2) ^ self.checksum_battery_voltage_before_dot) ^ self.checksum_battery_voltage_after_dot_0) ^ self.checksum_battery_voltage_after_dot_1) ^ self.checksum_no_of_resets_raw_0) ^ self.checksum_no_of_resets_raw_1) ^ self.checksum_no_of_resets_raw_2) ^ self.checksum_antenna_deployment_status_raw) ^ self.checksum_radiation_level_raw_0) ^ self.checksum_radiation_level_raw_1) ^ self.checksum_radiation_level_raw_2) ^ self.checksum_five_commata_and_one_dot)
            return self._m_checksum_calculation if hasattr(self, '_m_checksum_calculation') else None

        @property
        def antenna_deployment_status(self):
            if hasattr(self, '_m_antenna_deployment_status'):
                return self._m_antenna_deployment_status if hasattr(self, '_m_antenna_deployment_status') else None

            self._m_antenna_deployment_status = int(self.antenna_deployment_status_raw)
            return self._m_antenna_deployment_status if hasattr(self, '_m_antenna_deployment_status') else None

        @property
        def checksum_radiation_level_raw_2(self):
            if hasattr(self, '_m_checksum_radiation_level_raw_2'):
                return self._m_checksum_radiation_level_raw_2 if hasattr(self, '_m_checksum_radiation_level_raw_2') else None

            self._m_checksum_radiation_level_raw_2 = (0 if (self.radiation_level_raw)[2:3] == u"" else (int((self.radiation_level_raw)[2:3], 16) + 48))
            return self._m_checksum_radiation_level_raw_2 if hasattr(self, '_m_checksum_radiation_level_raw_2') else None

        @property
        def no_of_resets(self):
            if hasattr(self, '_m_no_of_resets'):
                return self._m_no_of_resets if hasattr(self, '_m_no_of_resets') else None

            self._m_no_of_resets = int(self.no_of_resets_raw)
            return self._m_no_of_resets if hasattr(self, '_m_no_of_resets') else None

        @property
        def checksum_battery_voltage_after_dot_0(self):
            if hasattr(self, '_m_checksum_battery_voltage_after_dot_0'):
                return self._m_checksum_battery_voltage_after_dot_0 if hasattr(self, '_m_checksum_battery_voltage_after_dot_0') else None

            self._m_checksum_battery_voltage_after_dot_0 = (int((self.battery_voltage_after_dot_raw)[0:1], 16) + 48)
            return self._m_checksum_battery_voltage_after_dot_0 if hasattr(self, '_m_checksum_battery_voltage_after_dot_0') else None

        @property
        def checksum_battery_temperature_1(self):
            if hasattr(self, '_m_checksum_battery_temperature_1'):
                return self._m_checksum_battery_temperature_1 if hasattr(self, '_m_checksum_battery_temperature_1') else None

            self._m_checksum_battery_temperature_1 = (0 if (self.battery_temperature_raw)[1:2] == u"" else (int((self.battery_temperature_raw)[1:2], 16) + 48))
            return self._m_checksum_battery_temperature_1 if hasattr(self, '_m_checksum_battery_temperature_1') else None

        @property
        def checksum_antenna_deployment_status_raw(self):
            if hasattr(self, '_m_checksum_antenna_deployment_status_raw'):
                return self._m_checksum_antenna_deployment_status_raw if hasattr(self, '_m_checksum_antenna_deployment_status_raw') else None

            self._m_checksum_antenna_deployment_status_raw = (int((self.antenna_deployment_status_raw)[0:1], 16) + 48)
            return self._m_checksum_antenna_deployment_status_raw if hasattr(self, '_m_checksum_antenna_deployment_status_raw') else None

        @property
        def checksum_no_of_resets_raw_2(self):
            if hasattr(self, '_m_checksum_no_of_resets_raw_2'):
                return self._m_checksum_no_of_resets_raw_2 if hasattr(self, '_m_checksum_no_of_resets_raw_2') else None

            self._m_checksum_no_of_resets_raw_2 = (0 if (self.no_of_resets_raw)[2:3] == u"" else (int((self.no_of_resets_raw)[2:3], 16) + 48))
            return self._m_checksum_no_of_resets_raw_2 if hasattr(self, '_m_checksum_no_of_resets_raw_2') else None

        @property
        def checksum_battery_temperature_3(self):
            if hasattr(self, '_m_checksum_battery_temperature_3'):
                return self._m_checksum_battery_temperature_3 if hasattr(self, '_m_checksum_battery_temperature_3') else None

            self._m_checksum_battery_temperature_3 = (0 if (self.battery_temperature_raw)[3:4] == u"" else (int((self.battery_temperature_raw)[3:4], 16) + 48))
            return self._m_checksum_battery_temperature_3 if hasattr(self, '_m_checksum_battery_temperature_3') else None

        @property
        def checksum_battery_temperature_0(self):
            if hasattr(self, '_m_checksum_battery_temperature_0'):
                return self._m_checksum_battery_temperature_0 if hasattr(self, '_m_checksum_battery_temperature_0') else None

            self._m_checksum_battery_temperature_0 = (45 if (self.battery_temperature_raw)[0:1] == u"-" else (int((self.battery_temperature_raw)[0:1], 16) + 48))
            return self._m_checksum_battery_temperature_0 if hasattr(self, '_m_checksum_battery_temperature_0') else None


    class DecisionDigiOrCw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check
            if _on == 65535:
                self.decision = Sanosat1.DigiWithDelimiter(self._io, self, self._root)
            elif _on == 19777:
                self.decision = Sanosat1.Cw(self._io, self, self._root)
            elif _on == 28001:
                self.decision = Sanosat1.Cw(self._io, self, self._root)
            else:
                self.decision = Sanosat1.DigiWithoutDelimiter(self._io, self, self._root)

        @property
        def check(self):
            if hasattr(self, '_m_check'):
                return self._m_check if hasattr(self, '_m_check') else None

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_check = self._io.read_u2le()
            self._io.seek(_pos)
            return self._m_check if hasattr(self, '_m_check') else None


    class WithoutDelimiter(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (KaitaiStream.bytes_terminate(self._io.read_bytes(7), 0, False)).decode(u"ASCII")
            if not  ((self.callsign == u"AM9NPQ")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/without_delimiter/seq/0")
            self.packet_type = self._io.read_s2le()
            self.com_temperature = self._io.read_s2le()
            self.battery_voltage = self._io.read_s2le()
            self.charging_current = self._io.read_s2le()
            self.battery_temperature = self._io.read_s2le()
            self.radiation_level = self._io.read_s2le()
            self.no_of_resets = self._io.read_s2le()
            self.antenna_deployment_status = self._io.read_u1()

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

            self._m_beacon_type = (u"GFSK" if 0 == 0 else u"GFSK")
            return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None


    class DigiWithoutDelimiter(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header = (self._io.read_bytes(3)).decode(u"ASCII")
            if not  ((self.header == u"NPQ")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.header, self._io, u"/types/digi_without_delimiter/seq/0")
            self.digimessage = (self._io.read_bytes_full()).decode(u"UTF-8")

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

            self._m_beacon_type = (u"DIGI" if 0 == 0 else u"DIGI")
            return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None


    class Cw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"AM9NPQ") or (self.callsign == u"am9npq")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/cw/seq/0")
            self.cwmessage = (self._io.read_bytes_term(63, False, True, True)).decode(u"UTF-8")
            self.checksum_cw_raw = (self._io.read_bytes(2)).decode(u"UTF-8")
            if not  ((self.checksum_cw_raw == self.checksum_compare)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.checksum_cw_raw, self._io, u"/types/cw/seq/2")

        @property
        def calc_checksum_cw_9(self):
            if hasattr(self, '_m_calc_checksum_cw_9'):
                return self._m_calc_checksum_cw_9 if hasattr(self, '_m_calc_checksum_cw_9') else None

            self._m_calc_checksum_cw_9 = (0 if (self.cwmessage)[9:10] == u"" else ((int((self.cwmessage)[9:10], 16) + 48) if int((self.cwmessage)[9:10], 16) < 10 else (int((self.cwmessage)[9:10], 16) + 87)))
            return self._m_calc_checksum_cw_9 if hasattr(self, '_m_calc_checksum_cw_9') else None

        @property
        def battery_temperature(self):
            if hasattr(self, '_m_battery_temperature'):
                return self._m_battery_temperature if hasattr(self, '_m_battery_temperature') else None

            self._m_battery_temperature = ((int((self.cwmessage)[self.length_of_obc_temp:(self.length_of_obc_temp + self.length_of_bat_temp)]) * -1) if self.sign_of_bat_temp == u"-" else int((self.cwmessage)[self.length_of_obc_temp:(self.length_of_obc_temp + self.length_of_bat_temp)]))
            return self._m_battery_temperature if hasattr(self, '_m_battery_temperature') else None

        @property
        def residuals(self):
            if hasattr(self, '_m_residuals'):
                return self._m_residuals if hasattr(self, '_m_residuals') else None

            self._m_residuals = int((self.cwmessage)[(len(self.cwmessage) - 2):len(self.cwmessage)], 16)
            return self._m_residuals if hasattr(self, '_m_residuals') else None

        @property
        def calc_checksum_cw_6(self):
            if hasattr(self, '_m_calc_checksum_cw_6'):
                return self._m_calc_checksum_cw_6 if hasattr(self, '_m_calc_checksum_cw_6') else None

            self._m_calc_checksum_cw_6 = ((int((self.cwmessage)[6:7], 16) + 48) if int((self.cwmessage)[6:7], 16) < 10 else (int((self.cwmessage)[6:7], 16) + 87))
            return self._m_calc_checksum_cw_6 if hasattr(self, '_m_calc_checksum_cw_6') else None

        @property
        def battery_voltage(self):
            if hasattr(self, '_m_battery_voltage'):
                return self._m_battery_voltage if hasattr(self, '_m_battery_voltage') else None

            self._m_battery_voltage = (int((self.cwmessage)[((self.length_of_obc_temp + self.length_of_bat_temp) + self.length_of_current):(((self.length_of_obc_temp + self.length_of_bat_temp) + self.length_of_current) + 2)]) * 10)
            return self._m_battery_voltage if hasattr(self, '_m_battery_voltage') else None

        @property
        def calc_checksum_cw_1(self):
            if hasattr(self, '_m_calc_checksum_cw_1'):
                return self._m_calc_checksum_cw_1 if hasattr(self, '_m_calc_checksum_cw_1') else None

            self._m_calc_checksum_cw_1 = ((int((self.cwmessage)[1:2], 16) + 48) if int((self.cwmessage)[1:2], 16) < 10 else (int((self.cwmessage)[1:2], 16) + 87))
            return self._m_calc_checksum_cw_1 if hasattr(self, '_m_calc_checksum_cw_1') else None

        @property
        def checksum_compare(self):
            if hasattr(self, '_m_checksum_compare'):
                return self._m_checksum_compare if hasattr(self, '_m_checksum_compare') else None

            self._m_checksum_compare = (self.checksum_cw_raw if self.checksum_calculation == int(self.checksum_cw_raw, 16) else u"checksum invalid")
            return self._m_checksum_compare if hasattr(self, '_m_checksum_compare') else None

        @property
        def sign_of_obc_temp(self):
            if hasattr(self, '_m_sign_of_obc_temp'):
                return self._m_sign_of_obc_temp if hasattr(self, '_m_sign_of_obc_temp') else None

            self._m_sign_of_obc_temp = (u"+" if ((self.residuals & 64) >> 6) == 0 else u"-")
            return self._m_sign_of_obc_temp if hasattr(self, '_m_sign_of_obc_temp') else None

        @property
        def calc_checksum_cw_4(self):
            if hasattr(self, '_m_calc_checksum_cw_4'):
                return self._m_calc_checksum_cw_4 if hasattr(self, '_m_calc_checksum_cw_4') else None

            self._m_calc_checksum_cw_4 = ((int((self.cwmessage)[4:5], 16) + 48) if int((self.cwmessage)[4:5], 16) < 10 else (int((self.cwmessage)[4:5], 16) + 87))
            return self._m_calc_checksum_cw_4 if hasattr(self, '_m_calc_checksum_cw_4') else None

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

            self._m_beacon_type = (u"CW" if 0 == 0 else u"CW")
            return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

        @property
        def calc_checksum_cw_3(self):
            if hasattr(self, '_m_calc_checksum_cw_3'):
                return self._m_calc_checksum_cw_3 if hasattr(self, '_m_calc_checksum_cw_3') else None

            self._m_calc_checksum_cw_3 = ((int((self.cwmessage)[3:4], 16) + 48) if int((self.cwmessage)[3:4], 16) < 10 else (int((self.cwmessage)[3:4], 16) + 87))
            return self._m_calc_checksum_cw_3 if hasattr(self, '_m_calc_checksum_cw_3') else None

        @property
        def calc_checksum_cw_5(self):
            if hasattr(self, '_m_calc_checksum_cw_5'):
                return self._m_calc_checksum_cw_5 if hasattr(self, '_m_calc_checksum_cw_5') else None

            self._m_calc_checksum_cw_5 = ((int((self.cwmessage)[5:6], 16) + 48) if int((self.cwmessage)[5:6], 16) < 10 else (int((self.cwmessage)[5:6], 16) + 87))
            return self._m_calc_checksum_cw_5 if hasattr(self, '_m_calc_checksum_cw_5') else None

        @property
        def calc_checksum_cw_7(self):
            if hasattr(self, '_m_calc_checksum_cw_7'):
                return self._m_calc_checksum_cw_7 if hasattr(self, '_m_calc_checksum_cw_7') else None

            self._m_calc_checksum_cw_7 = (0 if (self.cwmessage)[7:8] == u"" else ((int((self.cwmessage)[7:8], 16) + 48) if int((self.cwmessage)[7:8], 16) < 10 else (int((self.cwmessage)[7:8], 16) + 87)))
            return self._m_calc_checksum_cw_7 if hasattr(self, '_m_calc_checksum_cw_7') else None

        @property
        def calc_checksum_cw_11(self):
            if hasattr(self, '_m_calc_checksum_cw_11'):
                return self._m_calc_checksum_cw_11 if hasattr(self, '_m_calc_checksum_cw_11') else None

            self._m_calc_checksum_cw_11 = (0 if (self.cwmessage)[11:12] == u"" else ((int((self.cwmessage)[11:12], 16) + 48) if int((self.cwmessage)[11:12], 16) < 10 else (int((self.cwmessage)[11:12], 16) + 87)))
            return self._m_calc_checksum_cw_11 if hasattr(self, '_m_calc_checksum_cw_11') else None

        @property
        def calc_checksum_cw_2(self):
            if hasattr(self, '_m_calc_checksum_cw_2'):
                return self._m_calc_checksum_cw_2 if hasattr(self, '_m_calc_checksum_cw_2') else None

            self._m_calc_checksum_cw_2 = ((int((self.cwmessage)[2:3], 16) + 48) if int((self.cwmessage)[2:3], 16) < 10 else (int((self.cwmessage)[2:3], 16) + 87))
            return self._m_calc_checksum_cw_2 if hasattr(self, '_m_calc_checksum_cw_2') else None

        @property
        def length_of_obc_temp(self):
            if hasattr(self, '_m_length_of_obc_temp'):
                return self._m_length_of_obc_temp if hasattr(self, '_m_length_of_obc_temp') else None

            self._m_length_of_obc_temp = (((len(self.cwmessage) - 4) - self.length_of_current) - self.length_of_bat_temp)
            return self._m_length_of_obc_temp if hasattr(self, '_m_length_of_obc_temp') else None

        @property
        def charging_current(self):
            if hasattr(self, '_m_charging_current'):
                return self._m_charging_current if hasattr(self, '_m_charging_current') else None

            self._m_charging_current = int((self.cwmessage)[(self.length_of_obc_temp + self.length_of_bat_temp):((self.length_of_obc_temp + self.length_of_bat_temp) + self.length_of_current)])
            return self._m_charging_current if hasattr(self, '_m_charging_current') else None

        @property
        def calc_checksum_cw_10(self):
            if hasattr(self, '_m_calc_checksum_cw_10'):
                return self._m_calc_checksum_cw_10 if hasattr(self, '_m_calc_checksum_cw_10') else None

            self._m_calc_checksum_cw_10 = (0 if (self.cwmessage)[10:11] == u"" else ((int((self.cwmessage)[10:11], 16) + 48) if int((self.cwmessage)[10:11], 16) < 10 else (int((self.cwmessage)[10:11], 16) + 87)))
            return self._m_calc_checksum_cw_10 if hasattr(self, '_m_calc_checksum_cw_10') else None

        @property
        def length_of_bat_temp(self):
            if hasattr(self, '_m_length_of_bat_temp'):
                return self._m_length_of_bat_temp if hasattr(self, '_m_length_of_bat_temp') else None

            self._m_length_of_bat_temp = ((self.residuals & 3) >> 0)
            return self._m_length_of_bat_temp if hasattr(self, '_m_length_of_bat_temp') else None

        @property
        def checksum_calculation(self):
            if hasattr(self, '_m_checksum_calculation'):
                return self._m_checksum_calculation if hasattr(self, '_m_checksum_calculation') else None

            self._m_checksum_calculation = ((((((((((((self.calc_checksum_cw_0 ^ self.calc_checksum_cw_1) ^ self.calc_checksum_cw_2) ^ self.calc_checksum_cw_3) ^ self.calc_checksum_cw_4) ^ self.calc_checksum_cw_5) ^ self.calc_checksum_cw_6) ^ self.calc_checksum_cw_7) ^ self.calc_checksum_cw_8) ^ self.calc_checksum_cw_9) ^ self.calc_checksum_cw_10) ^ self.calc_checksum_cw_11) ^ self.calc_checksum_cw_12)
            return self._m_checksum_calculation if hasattr(self, '_m_checksum_calculation') else None

        @property
        def antenna_deployment_status(self):
            if hasattr(self, '_m_antenna_deployment_status'):
                return self._m_antenna_deployment_status if hasattr(self, '_m_antenna_deployment_status') else None

            self._m_antenna_deployment_status = (1 if ((self.residuals & 128) >> 7) == 1 else 0)
            return self._m_antenna_deployment_status if hasattr(self, '_m_antenna_deployment_status') else None

        @property
        def calc_checksum_cw_12(self):
            if hasattr(self, '_m_calc_checksum_cw_12'):
                return self._m_calc_checksum_cw_12 if hasattr(self, '_m_calc_checksum_cw_12') else None

            self._m_calc_checksum_cw_12 = (0 if (self.cwmessage)[12:13] == u"" else ((int((self.cwmessage)[12:13], 16) + 48) if int((self.cwmessage)[12:13], 16) < 10 else (int((self.cwmessage)[12:13], 16) + 87)))
            return self._m_calc_checksum_cw_12 if hasattr(self, '_m_calc_checksum_cw_12') else None

        @property
        def length_of_current(self):
            if hasattr(self, '_m_length_of_current'):
                return self._m_length_of_current if hasattr(self, '_m_length_of_current') else None

            self._m_length_of_current = ((self.residuals & 28) >> 2)
            return self._m_length_of_current if hasattr(self, '_m_length_of_current') else None

        @property
        def sign_of_bat_temp(self):
            if hasattr(self, '_m_sign_of_bat_temp'):
                return self._m_sign_of_bat_temp if hasattr(self, '_m_sign_of_bat_temp') else None

            self._m_sign_of_bat_temp = (u"+" if ((self.residuals & 32) >> 5) == 0 else u"-")
            return self._m_sign_of_bat_temp if hasattr(self, '_m_sign_of_bat_temp') else None

        @property
        def calc_checksum_cw_8(self):
            if hasattr(self, '_m_calc_checksum_cw_8'):
                return self._m_calc_checksum_cw_8 if hasattr(self, '_m_calc_checksum_cw_8') else None

            self._m_calc_checksum_cw_8 = (0 if (self.cwmessage)[8:9] == u"" else ((int((self.cwmessage)[8:9], 16) + 48) if int((self.cwmessage)[8:9], 16) < 10 else (int((self.cwmessage)[8:9], 16) + 87)))
            return self._m_calc_checksum_cw_8 if hasattr(self, '_m_calc_checksum_cw_8') else None

        @property
        def com_temperature(self):
            if hasattr(self, '_m_com_temperature'):
                return self._m_com_temperature if hasattr(self, '_m_com_temperature') else None

            self._m_com_temperature = ((int((self.cwmessage)[0:self.length_of_obc_temp]) * -1) if self.sign_of_obc_temp == u"-" else int((self.cwmessage)[0:self.length_of_obc_temp]))
            return self._m_com_temperature if hasattr(self, '_m_com_temperature') else None

        @property
        def calc_checksum_cw_0(self):
            if hasattr(self, '_m_calc_checksum_cw_0'):
                return self._m_calc_checksum_cw_0 if hasattr(self, '_m_calc_checksum_cw_0') else None

            self._m_calc_checksum_cw_0 = ((int((self.cwmessage)[0:1], 16) + 48) if int((self.cwmessage)[0:1], 16) < 10 else (int((self.cwmessage)[0:1], 16) + 87))
            return self._m_calc_checksum_cw_0 if hasattr(self, '_m_calc_checksum_cw_0') else None


    class Sanosat1TelemetryT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check
            if _on == 2606547689692286273:
                self.body = Sanosat1.Rtty(self._io, self, self._root)
            elif _on == 2606583012040207713:
                self.body = Sanosat1.Rtty(self._io, self, self._root)
            elif _on == 5636621350199164927:
                self.body = Sanosat1.WithDelimiter(self._io, self, self._root)
            elif _on == 72146999389539649:
                self.body = Sanosat1.WithoutDelimiter(self._io, self, self._root)
            else:
                self.body = Sanosat1.DecisionDigiOrCw(self._io, self, self._root)

        @property
        def check(self):
            if hasattr(self, '_m_check'):
                return self._m_check if hasattr(self, '_m_check') else None

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_check = self._io.read_u8le()
            self._io.seek(_pos)
            return self._m_check if hasattr(self, '_m_check') else None


    class DigiWithDelimiter(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.delimiter = self._io.read_s4le()
            if not  ((self.delimiter == 65535)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.delimiter, self._io, u"/types/digi_with_delimiter/seq/0")
            self.header = (self._io.read_bytes(3)).decode(u"ASCII")
            if not  ((self.header == u"NPQ")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.header, self._io, u"/types/digi_with_delimiter/seq/1")
            self.digimessage = (self._io.read_bytes_full()).decode(u"UTF-8")

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

            self._m_beacon_type = (u"DIGI" if 0 == 0 else u"DIGI")
            return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Lucky7(KaitaiStruct):
    """:field obc_id: obc_id
    :field mission_counter: mission_counter
    :field callsign_and_satellite_name: callsign_and_satellite_name
    :field total_reset_counter: total_reset_counter
    :field swap_reset_counter: swap_reset_counter
    :field battery_voltage: battery_voltage
    :field mcu_temperature: mcu_temperature
    :field pa_temperature: pa_temperature
    :field processor_current: processor_current
    :field mcu_voltage_3v3: mcu_voltage_3v3
    :field mcu_voltage_1v2: mcu_voltage_1v2
    :field angular_rate_x_axis: angular_rate_x_axis
    :field angular_rate_y_axis: angular_rate_y_axis
    :field angular_rate_z_axis: angular_rate_z_axis
    :field antenna_burnwire: antenna_burnwire
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.obc_id = self._io.read_s2le()
        self.mission_counter = self._io.read_u4be()
        self.callsign_and_satellite_name = (self._io.read_bytes(12)).decode(u"ASCII")
        if not self.callsign_and_satellite_name == u"OK0SATLUCKY7":
            raise kaitaistruct.ValidationNotEqualError(u"OK0SATLUCKY7", self.callsign_and_satellite_name, self._io, u"/seq/2")
        self.total_reset_counter = self._io.read_u2be()
        self.swap_reset_counter = self._io.read_u2be()
        self.battery_voltage = self._io.read_u1()
        self.mcu_temperature = self._io.read_s1()
        self.pa_temperature = self._io.read_s1()
        self.processor_current = self._io.read_u1()
        self.mcu_voltage_3v3 = self._io.read_u1()
        self.mcu_voltage_1v2 = self._io.read_u1()
        self.angular_rate_x_axis = self._io.read_s2be()
        self.angular_rate_y_axis = self._io.read_s2be()
        self.angular_rate_z_axis = self._io.read_s2be()
        self.antenna_burnwire = self._io.read_u1()





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
from enum import Enum


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Celesta(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.dest_callsign_ror.dest_callsign
    :field dest_ssid_raw: ax25_frame.ax25_header.dest_ssid_raw
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.src_callsign_ror.src_callsign
    :field src_ssid_raw: ax25_frame.ax25_header.src_ssid_raw
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.ax25_data.pid
    :field size: ax25_frame.ax25_data.size
    :field frame_type: ax25_frame.ax25_data.frame_type
    :field ts: ax25_frame.ax25_data.ts
    :field timestamp: ax25_frame.ax25_data.obdh.timestamp
    :field temperature: ax25_frame.ax25_data.obdh.temperature
    :field satellite_mode: ax25_frame.ax25_data.obdh.satellite_mode
    :field obdh_mode: ax25_frame.ax25_data.obdh.obdh_mode
    :field bytes_to_transmit: ax25_frame.ax25_data.obdh.bytes_to_transmit
    :field number_of_resets: ax25_frame.ax25_data.obdh.number_of_resets
    :field number_of_errors: ax25_frame.ax25_data.obdh.number_of_errors
    :field eps_mode: ax25_frame.ax25_data.eps.eps_mode
    :field battery_voltage_raw: ax25_frame.ax25_data.eps.battery_voltage_raw
    :field battery_temperature: ax25_frame.ax25_data.eps.battery_temperature
    :field min_battery_voltage_raw: ax25_frame.ax25_data.eps.min_battery_voltage_raw
    :field max_battery_voltage_raw: ax25_frame.ax25_data.eps.max_battery_voltage_raw
    :field avg_battery_voltage_raw: ax25_frame.ax25_data.eps.avg_battery_voltage_raw
    :field avg_charge_current_raw: ax25_frame.ax25_data.eps.avg_charge_current_raw
    :field max_charge_current_raw: ax25_frame.ax25_data.eps.max_charge_current_raw
    :field z_minu_face_temperature: ax25_frame.ax25_data.eps.z_minu_face_temperature
    :field o_b_d_h_current: ax25_frame.ax25_data.eps.o_b_d_h_current
    :field e_p_s_current: ax25_frame.ax25_data.eps.e_p_s_current
    :field t_t_c_micro_c_current: ax25_frame.ax25_data.eps.t_t_c_micro_c_current
    :field t_t_c_p_a_current_raw: ax25_frame.ax25_data.eps.t_t_c_p_a_current_raw
    :field d_o_s_i_current: ax25_frame.ax25_data.eps.d_o_s_i_current
    :field charge_current_raw: ax25_frame.ax25_data.eps.charge_current_raw
    :field spare: ax25_frame.ax25_data.eps.spare
    :field last_battery_voltage: ax25_frame.ax25_data.eps.last_battery_voltage
    :field minimum_battery_voltage_measured_since_reboot: ax25_frame.ax25_data.eps.minimum_battery_voltage_measured_since_reboot
    :field maximum_battery_voltage_measured_since_reboot: ax25_frame.ax25_data.eps.maximum_battery_voltage_measured_since_reboot
    :field average_battery_voltage_measured_since_reboot: ax25_frame.ax25_data.eps.average_battery_voltage_measured_since_reboot
    :field average_charge_current_measured_since_reboot: ax25_frame.ax25_data.eps.average_charge_current_measured_since_reboot
    :field maximum_charge_current_measured_since_reboot: ax25_frame.ax25_data.eps.maximum_charge_current_measured_since_reboot
    :field current_consumption_of_the_power_amplifier_of_the_ttc: ax25_frame.ax25_data.eps.current_consumption_of_the_power_amplifier_of_the_ttc
    :field total_charge_current_of_the_battery: ax25_frame.ax25_data.eps.total_charge_current_of_the_battery
    :field mode: ax25_frame.ax25_data.ttc.mode
    :field number_of_ttc_resets: ax25_frame.ax25_data.ttc.number_of_ttc_resets
    :field last_reset_cause: ax25_frame.ax25_data.ttc.last_reset_cause
    :field number_of_received_valid_packets: ax25_frame.ax25_data.ttc.number_of_received_valid_packets
    :field number_of_transmitted_packets: ax25_frame.ax25_data.ttc.number_of_transmitted_packets
    :field measured_transmission_power: ax25_frame.ax25_data.ttc.measured_transmission_power
    :field last_error_code: ax25_frame.ax25_data.ttc.last_error_code
    :field power_configuration: ax25_frame.ax25_data.ttc.power_configuration
    :field power_amplifier_temperature: ax25_frame.ax25_data.ttc.power_amplifier_temperature
    :field rssi_of_last_received_packet_raw: ax25_frame.ax25_data.ttc.rssi_of_last_received_packet_raw
    :field frequency_deviation_of_last_received_packet_raw: ax25_frame.ax25_data.ttc.frequency_deviation_of_last_received_packet_raw
    :field beacon_period: ax25_frame.ax25_data.ttc.beacon_period
    :field rssi_of_last_received_packet: ax25_frame.ax25_data.ttc.rssi_of_last_received_packet
    :field frequency_deviation_of_last_received_packet_with_valid_crc: ax25_frame.ax25_data.ttc.frequency_deviation_of_last_received_packet_with_valid_crc
    :field last_message_rssi_raw: ax25_frame.ax25_data.ham_message.last_message_rssi_raw
    :field radio_message: ax25_frame.ax25_data.ham_message.radio_message
    :field last_msg_rssi_dbm: ax25_frame.ax25_data.ham_message.last_msg_rssi_dbm
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Celesta.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Celesta.Ax25Header(self._io, self, self._root)
            self.ax25_data = Celesta.Ax25Data(self._io, self, self._root)


    class DestCallsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Celesta.DestCallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = self._io.read_u1()
            self.src_callsign_raw = Celesta.SrcCallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = self._io.read_u1()
            self.ctl = self._io.read_u1()


    class HamMessage(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.last_message_rssi_raw = self._io.read_u1()
            self.radio_message = (self._io.read_bytes(133)).decode(u"ASCII")

        @property
        def last_msg_rssi_dbm(self):
            if hasattr(self, '_m_last_msg_rssi_dbm'):
                return self._m_last_msg_rssi_dbm if hasattr(self, '_m_last_msg_rssi_dbm') else None

            self._m_last_msg_rssi_dbm = (-1 * self.last_message_rssi_raw)
            return self._m_last_msg_rssi_dbm if hasattr(self, '_m_last_msg_rssi_dbm') else None


    class SrcCallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_src_callsign_ror = self._io.read_bytes(6)
            self._raw_src_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_src_callsign_ror, 8 - (1), 1)
            _io__raw_src_callsign_ror = KaitaiStream(BytesIO(self._raw_src_callsign_ror))
            self.src_callsign_ror = Celesta.SrcCallsign(_io__raw_src_callsign_ror, self, self._root)


    class Ttc(KaitaiStruct):

        class EMode(Enum):
            idle = 1
            beacon = 17
            commissionning = 34
            silent = 68

        class ELastResetCause(Enum):
            power_supply_reset = 17
            watchdog = 34
            oscillator_error = 51
            reset_pin = 68
            debugger_reset = 85
            software_reset = 119

        class ELastErrorCode(Enum):
            nothing = 0
            obdh_status_req = 1
            obdh_bdr_req = 2
            radio_hw_error = 17
            tx_queue_full = 34
            rx_queue_full = 51
            tx_bus_queue_full = 68
            rx_bus_queue_full = 85
            obc_temp_hw_error = 102
            obc_temp_h_limit_error = 119
            obc_temp_l_limit_error = 136
            pa_temp_hw_error = 153
            fram_id_error = 161
            fram_hw_error = 162
            fram_read_error = 163
            fram_write_error = 164
            event_queue_read_error = 165
            pa_temp_h_limit_error = 170
            pa_temp_l_limit_error = 187
            obdh_nack = 204
            ttc_reset_req = 209
            pf_reset_req = 221
            radio_task_timeout = 238
            radio_unqueue = 255
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.mode = KaitaiStream.resolve_enum(Celesta.Ttc.EMode, self._io.read_u1())
            self.number_of_ttc_resets = self._io.read_u2le()
            self.last_reset_cause = KaitaiStream.resolve_enum(Celesta.Ttc.ELastResetCause, self._io.read_u1())
            self.number_of_received_valid_packets = self._io.read_u2le()
            self.number_of_transmitted_packets = self._io.read_u2le()
            self.measured_transmission_power = self._io.read_u2le()
            self.last_error_code = KaitaiStream.resolve_enum(Celesta.Ttc.ELastErrorCode, self._io.read_u1())
            self.power_configuration = self._io.read_u1()
            self.power_amplifier_temperature = self._io.read_s1()
            self.rssi_of_last_received_packet_raw = self._io.read_u1()
            self.frequency_deviation_of_last_received_packet_raw = self._io.read_u1()
            self.beacon_period = self._io.read_u1()

        @property
        def rssi_of_last_received_packet(self):
            if hasattr(self, '_m_rssi_of_last_received_packet'):
                return self._m_rssi_of_last_received_packet if hasattr(self, '_m_rssi_of_last_received_packet') else None

            self._m_rssi_of_last_received_packet = (-1 * self.rssi_of_last_received_packet_raw)
            return self._m_rssi_of_last_received_packet if hasattr(self, '_m_rssi_of_last_received_packet') else None

        @property
        def frequency_deviation_of_last_received_packet_with_valid_crc(self):
            if hasattr(self, '_m_frequency_deviation_of_last_received_packet_with_valid_crc'):
                return self._m_frequency_deviation_of_last_received_packet_with_valid_crc if hasattr(self, '_m_frequency_deviation_of_last_received_packet_with_valid_crc') else None

            self._m_frequency_deviation_of_last_received_packet_with_valid_crc = (self.frequency_deviation_of_last_received_packet_raw * 17)
            return self._m_frequency_deviation_of_last_received_packet_with_valid_crc if hasattr(self, '_m_frequency_deviation_of_last_received_packet_with_valid_crc') else None


    class Obdh(KaitaiStruct):

        class ESatelliteMode(Enum):
            standby = 0
            deploy = 1
            commissionning = 2
            comm_pl = 3
            mission = 4
            low_p_mission = 5
            transmit = 6
            survival = 7
            silent = 8

        class EObdhMode(Enum):
            standby = 17
            deploy = 34
            commissionning = 51
            comm_pl = 68
            mission = 85
            low_power_mission = 102
            silent = 119
            por = 255
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = self._io.read_u4be()
            self.temperature = self._io.read_u2le()
            self.satellite_mode = KaitaiStream.resolve_enum(Celesta.Obdh.ESatelliteMode, self._io.read_u1())
            self.obdh_mode = KaitaiStream.resolve_enum(Celesta.Obdh.EObdhMode, self._io.read_u1())
            self.bytes_to_transmit = self._io.read_u4le()
            self.number_of_resets = self._io.read_u2le()
            self.number_of_errors = self._io.read_u2le()


    class Eps(KaitaiStruct):

        class EEpsMode(Enum):
            idle = 0
            survival = 17
            stnadby = 34
            deploy = 51
            commissionnong = 68
            mission = 85
            low_power_mission = 102
            silent = 119
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.eps_mode = KaitaiStream.resolve_enum(Celesta.Eps.EEpsMode, self._io.read_u1())
            self.battery_voltage_raw = self._io.read_u1()
            self.battery_temperature = self._io.read_s1()
            self.min_battery_voltage_raw = self._io.read_u1()
            self.max_battery_voltage_raw = self._io.read_u1()
            self.avg_battery_voltage_raw = self._io.read_u1()
            self.avg_charge_current_raw = self._io.read_u1()
            self.max_charge_current_raw = self._io.read_u1()
            self.z_minu_face_temperature = self._io.read_s1()
            self.o_b_d_h_current = self._io.read_u1()
            self.e_p_s_current = self._io.read_u1()
            self.t_t_c_micro_c_current = self._io.read_u1()
            self.t_t_c_p_a_current_raw = self._io.read_u1()
            self.d_o_s_i_current = self._io.read_u1()
            self.charge_current_raw = self._io.read_u1()
            self.spare = self._io.read_u1()

        @property
        def total_charge_current_of_the_battery(self):
            if hasattr(self, '_m_total_charge_current_of_the_battery'):
                return self._m_total_charge_current_of_the_battery if hasattr(self, '_m_total_charge_current_of_the_battery') else None

            self._m_total_charge_current_of_the_battery = (self.charge_current_raw * 12)
            return self._m_total_charge_current_of_the_battery if hasattr(self, '_m_total_charge_current_of_the_battery') else None

        @property
        def current_consumption_of_the_power_amplifier_of_the_ttc(self):
            if hasattr(self, '_m_current_consumption_of_the_power_amplifier_of_the_ttc'):
                return self._m_current_consumption_of_the_power_amplifier_of_the_ttc if hasattr(self, '_m_current_consumption_of_the_power_amplifier_of_the_ttc') else None

            self._m_current_consumption_of_the_power_amplifier_of_the_ttc = (self.t_t_c_p_a_current_raw * 5)
            return self._m_current_consumption_of_the_power_amplifier_of_the_ttc if hasattr(self, '_m_current_consumption_of_the_power_amplifier_of_the_ttc') else None

        @property
        def average_charge_current_measured_since_reboot(self):
            if hasattr(self, '_m_average_charge_current_measured_since_reboot'):
                return self._m_average_charge_current_measured_since_reboot if hasattr(self, '_m_average_charge_current_measured_since_reboot') else None

            self._m_average_charge_current_measured_since_reboot = (self.avg_charge_current_raw * 12)
            return self._m_average_charge_current_measured_since_reboot if hasattr(self, '_m_average_charge_current_measured_since_reboot') else None

        @property
        def maximum_charge_current_measured_since_reboot(self):
            if hasattr(self, '_m_maximum_charge_current_measured_since_reboot'):
                return self._m_maximum_charge_current_measured_since_reboot if hasattr(self, '_m_maximum_charge_current_measured_since_reboot') else None

            self._m_maximum_charge_current_measured_since_reboot = (self.max_charge_current_raw * 12)
            return self._m_maximum_charge_current_measured_since_reboot if hasattr(self, '_m_maximum_charge_current_measured_since_reboot') else None

        @property
        def maximum_battery_voltage_measured_since_reboot(self):
            if hasattr(self, '_m_maximum_battery_voltage_measured_since_reboot'):
                return self._m_maximum_battery_voltage_measured_since_reboot if hasattr(self, '_m_maximum_battery_voltage_measured_since_reboot') else None

            self._m_maximum_battery_voltage_measured_since_reboot = (self.max_battery_voltage_raw * 20)
            return self._m_maximum_battery_voltage_measured_since_reboot if hasattr(self, '_m_maximum_battery_voltage_measured_since_reboot') else None

        @property
        def last_battery_voltage(self):
            if hasattr(self, '_m_last_battery_voltage'):
                return self._m_last_battery_voltage if hasattr(self, '_m_last_battery_voltage') else None

            self._m_last_battery_voltage = (self.battery_voltage_raw * 20)
            return self._m_last_battery_voltage if hasattr(self, '_m_last_battery_voltage') else None

        @property
        def minimum_battery_voltage_measured_since_reboot(self):
            if hasattr(self, '_m_minimum_battery_voltage_measured_since_reboot'):
                return self._m_minimum_battery_voltage_measured_since_reboot if hasattr(self, '_m_minimum_battery_voltage_measured_since_reboot') else None

            self._m_minimum_battery_voltage_measured_since_reboot = (self.min_battery_voltage_raw * 20)
            return self._m_minimum_battery_voltage_measured_since_reboot if hasattr(self, '_m_minimum_battery_voltage_measured_since_reboot') else None

        @property
        def average_battery_voltage_measured_since_reboot(self):
            if hasattr(self, '_m_average_battery_voltage_measured_since_reboot'):
                return self._m_average_battery_voltage_measured_since_reboot if hasattr(self, '_m_average_battery_voltage_measured_since_reboot') else None

            self._m_average_battery_voltage_measured_since_reboot = (self.avg_battery_voltage_raw * 20)
            return self._m_average_battery_voltage_measured_since_reboot if hasattr(self, '_m_average_battery_voltage_measured_since_reboot') else None


    class DestCallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_dest_callsign_ror = self._io.read_bytes(6)
            self._raw_dest_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_dest_callsign_ror, 8 - (1), 1)
            _io__raw_dest_callsign_ror = KaitaiStream(BytesIO(self._raw_dest_callsign_ror))
            self.dest_callsign_ror = Celesta.DestCallsign(_io__raw_dest_callsign_ror, self, self._root)


    class Ax25Data(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.size = self._io.read_u1()
            self.frame_type = self._io.read_u1()
            self.ts = self._io.read_u4be()
            self.obdh = Celesta.Obdh(self._io, self, self._root)
            self.eps = Celesta.Eps(self._io, self, self._root)
            self.ttc = Celesta.Ttc(self._io, self, self._root)
            self.payload = self._io.read_bytes(48)
            self.ham_message = Celesta.HamMessage(self._io, self, self._root)


    class SrcCallsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.src_callsign = (self._io.read_bytes(6)).decode(u"ASCII")






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Cubebel2(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field beacon_trx_id: ax25_frame.payload.ax25_info.trx_beacon.beacon_id
    :field beacon_uptime: ax25_frame.payload.ax25_info.trx_beacon.beacon_uptime
    :field beacon_vbus: ax25_frame.payload.ax25_info.trx_beacon.beacon_vbus
    :field beacon_reset_total_cnt: ax25_frame.payload.ax25_info.trx_beacon.beacon_reset_total_cnt
    :field beacon_reset_iwdg_cnt: ax25_frame.payload.ax25_info.trx_beacon.beacon_reset_iwdg_cnt
    :field beacon_reset_iwdg_time: ax25_frame.payload.ax25_info.trx_beacon.beacon_reset_iwdg_time
    :field beacon_pamp_temp: ax25_frame.payload.ax25_info.trx_beacon.beacon_pamp_temp
    :field beacon_rx_settings: ax25_frame.payload.ax25_info.trx_beacon.beacon_rx_settings
    :field beacon_rx_period_on: ax25_frame.payload.ax25_info.trx_beacon.beacon_rx_period_on
    :field beacon_rx_seqnum: ax25_frame.payload.ax25_info.trx_beacon.beacon_rx_seqnum
    :field beacon_rx_cnt_total: ax25_frame.payload.ax25_info.trx_beacon.beacon_rx_cnt_total
    :field beacon_rx_cnt_valid: ax25_frame.payload.ax25_info.trx_beacon.beacon_rx_cnt_valid
    :field beacon_tx_settings: ax25_frame.payload.ax25_info.trx_beacon.beacon_tx_settings
    :field beacon_tx_pwr: ax25_frame.payload.ax25_info.trx_beacon.beacon_tx_pwr
    :field beacon_tx_cnt: ax25_frame.payload.ax25_info.trx_beacon.beacon_tx_cnt
    :field cdm_id: ax25_frame.payload.ax25_info.cdm_header.cdm_id
    :field cdm_addr_src: ax25_frame.payload.ax25_info.cdm_header.cdm_addr_src
    :field config_mb_1: ax25_frame.payload.ax25_info.cdm_payload.record_mb.0.value
    :field config_mb_2: ax25_frame.payload.ax25_info.cdm_payload.record_mb.1.value
    :field config_mb_3: ax25_frame.payload.ax25_info.cdm_payload.record_mb.2.value
    :field config_mb_4: ax25_frame.payload.ax25_info.cdm_payload.record_mb.3.value
    :field config_mb_5: ax25_frame.payload.ax25_info.cdm_payload.record_mb.4.value
    :field config_mb_6: ax25_frame.payload.ax25_info.cdm_payload.record_mb.5.value
    :field config_mb_7: ax25_frame.payload.ax25_info.cdm_payload.record_mb.6.value
    :field config_mb_8: ax25_frame.payload.ax25_info.cdm_payload.record_mb.7.value
    :field config_mb_9: ax25_frame.payload.ax25_info.cdm_payload.record_mb.8.value
    :field config_mb_10: ax25_frame.payload.ax25_info.cdm_payload.record_mb.9.value
    :field config_mb_11: ax25_frame.payload.ax25_info.cdm_payload.record_mb.10.value
    :field config_mb_12: ax25_frame.payload.ax25_info.cdm_payload.record_mb.11.value
    :field config_mb_13: ax25_frame.payload.ax25_info.cdm_payload.record_mb.12.value
    :field config_mb_14: ax25_frame.payload.ax25_info.cdm_payload.record_mb.13.value
    :field config_mb_15: ax25_frame.payload.ax25_info.cdm_payload.record_mb.14.value
    :field config_mb_16: ax25_frame.payload.ax25_info.cdm_payload.record_mb.15.value
    :field config_mb_17: ax25_frame.payload.ax25_info.cdm_payload.record_mb.16.value
    :field config_mb_18: ax25_frame.payload.ax25_info.cdm_payload.record_mb.17.value
    :field config_mb_19: ax25_frame.payload.ax25_info.cdm_payload.record_mb.18.value
    :field config_trx_1: ax25_frame.payload.ax25_info.cdm_payload.record_trx.0.value
    :field config_trx_2: ax25_frame.payload.ax25_info.cdm_payload.record_trx.1.value
    :field config_trx_3: ax25_frame.payload.ax25_info.cdm_payload.record_trx.2.value
    :field config_trx_4: ax25_frame.payload.ax25_info.cdm_payload.record_trx.3.value
    :field config_trx_5: ax25_frame.payload.ax25_info.cdm_payload.record_trx.4.value
    :field config_trx_6: ax25_frame.payload.ax25_info.cdm_payload.record_trx.5.value
    :field config_trx_7: ax25_frame.payload.ax25_info.cdm_payload.record_trx.6.value
    :field config_trx_8: ax25_frame.payload.ax25_info.cdm_payload.record_trx.7.value
    :field config_trx_9: ax25_frame.payload.ax25_info.cdm_payload.record_trx.8.value
    :field config_trx_10: ax25_frame.payload.ax25_info.cdm_payload.record_trx.9.value
    :field config_trx_11: ax25_frame.payload.ax25_info.cdm_payload.record_trx.10.value
    :field config_trx_12: ax25_frame.payload.ax25_info.cdm_payload.record_trx.11.value
    :field config_trx_13: ax25_frame.payload.ax25_info.cdm_payload.record_trx.12.value
    :field config_trx_14: ax25_frame.payload.ax25_info.cdm_payload.record_trx.13.value
    :field config_trx_15: ax25_frame.payload.ax25_info.cdm_payload.record_trx.14.value
    :field config_trx_16: ax25_frame.payload.ax25_info.cdm_payload.record_trx.15.value
    :field config_trx_17: ax25_frame.payload.ax25_info.cdm_payload.record_trx.16.value
    :field config_trx_18: ax25_frame.payload.ax25_info.cdm_payload.record_trx.17.value
    :field config_trx_19: ax25_frame.payload.ax25_info.cdm_payload.record_trx.18.value
    :field config_trx_20: ax25_frame.payload.ax25_info.cdm_payload.record_trx.19.value
    :field config_trx_21: ax25_frame.payload.ax25_info.cdm_payload.record_trx.20.value
    :field config_trx_22: ax25_frame.payload.ax25_info.cdm_payload.record_trx.21.value
    :field config_trx_23: ax25_frame.payload.ax25_info.cdm_payload.record_trx.22.value
    :field config_trx_24: ax25_frame.payload.ax25_info.cdm_payload.record_trx.23.value
    :field config_trx_25: ax25_frame.payload.ax25_info.cdm_payload.record_trx.24.value
    :field config_trx_26: ax25_frame.payload.ax25_info.cdm_payload.record_trx.25.value
    :field fwver_githash: ax25_frame.payload.ax25_info.cdm_payload.version_githash
    :field rfreply_rssi: ax25_frame.payload.ax25_info.cdm_payload.ax5043_rssi
    :field rfreply_agc: ax25_frame.payload.ax25_info.cdm_payload.ax5043_agc
    :field rfreply_seq_enabled: ax25_frame.payload.ax25_info.cdm_payload.command_seq_enabled
    :field rfreply_seq_valid: ax25_frame.payload.ax25_info.cdm_payload.command_seq_valid
    :field rfreply_seq_num: ax25_frame.payload.ax25_info.cdm_payload.command_seq_num
    :field tlm_mb_time: ax25_frame.payload.ax25_info.cdm_payload.time
    :field tlm_mb_mcusr: ax25_frame.payload.ax25_info.cdm_payload.mcusr
    :field tlm_mb_rst_cnt_total: ax25_frame.payload.ax25_info.cdm_payload.rst_cnt_total
    :field tlm_mb_rst_cnt_iwdg: ax25_frame.payload.ax25_info.cdm_payload.rst_cnt_iwdg
    :field tlm_mb_adc_status: ax25_frame.payload.ax25_info.cdm_payload.adc_status
    :field tlm_mb_adc_temp_1: ax25_frame.payload.ax25_info.cdm_payload.adc_temp_1
    :field tlm_mb_adc_temp_2: ax25_frame.payload.ax25_info.cdm_payload.adc_temp_2
    :field tlm_mb_ant_1_v: ax25_frame.payload.ax25_info.cdm_payload.ant_1_v
    :field tlm_mb_ant_2_v: ax25_frame.payload.ax25_info.cdm_payload.ant_2_v
    :field tlm_mb_solar_common_v: ax25_frame.payload.ax25_info.cdm_payload.solar_common_v
    :field tlm_mb_sat_bus_v: ax25_frame.payload.ax25_info.cdm_payload.sat_bus_v
    :field tlm_mb_sat_bus_c: ax25_frame.payload.ax25_info.cdm_payload.sat_bus_c
    :field tlm_mb_uc_v: ax25_frame.payload.ax25_info.cdm_payload.uc_v
    :field tlm_mb_uc_c: ax25_frame.payload.ax25_info.cdm_payload.uc_c
    :field tlm_mb_battpack_0_voltage: ax25_frame.payload.ax25_info.cdm_payload.battery_back.0.voltage
    :field tlm_mb_battpack_0_element_0_current: ax25_frame.payload.ax25_info.cdm_payload.battery_back.0.element.0.current
    :field tlm_mb_battpack_0_element_0_temp: ax25_frame.payload.ax25_info.cdm_payload.battery_back.0.element.0.temp
    :field tlm_mb_battpack_0_element_1_current: ax25_frame.payload.ax25_info.cdm_payload.battery_back.0.element.1.current
    :field tlm_mb_battpack_0_element_1_temp: ax25_frame.payload.ax25_info.cdm_payload.battery_back.0.element.1.temp
    :field tlm_mb_battpack_1_voltage: ax25_frame.payload.ax25_info.cdm_payload.battery_back.1.voltage
    :field tlm_mb_battpack_1_element_0_current: ax25_frame.payload.ax25_info.cdm_payload.battery_back.1.element.0.current
    :field tlm_mb_battpack_1_element_0_temp: ax25_frame.payload.ax25_info.cdm_payload.battery_back.1.element.0.temp
    :field tlm_mb_battpack_1_element_1_current: ax25_frame.payload.ax25_info.cdm_payload.battery_back.1.element.1.current
    :field tlm_mb_battpack_1_element_1_temp: ax25_frame.payload.ax25_info.cdm_payload.battery_back.1.element.1.temp
    :field tlm_mb_solarpanel_0_current: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.0.current
    :field tlm_mb_solarpanel_0_volt_pos: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.0.volt_pos
    :field tlm_mb_solarpanel_0_volt_neg: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.0.volt_neg
    :field tlm_mb_solarpanel_1_current: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.1.current
    :field tlm_mb_solarpanel_1_volt_pos: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.1.volt_pos
    :field tlm_mb_solarpanel_1_volt_neg: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.1.volt_neg
    :field tlm_mb_solarpanel_2_current: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.2.current
    :field tlm_mb_solarpanel_2_volt_pos: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.2.volt_pos
    :field tlm_mb_solarpanel_2_volt_neg: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.2.volt_neg
    :field tlm_mb_solarpanel_3_current: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.3.current
    :field tlm_mb_solarpanel_3_volt_pos: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.3.volt_pos
    :field tlm_mb_solarpanel_3_volt_neg: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.3.volt_neg
    :field tlm_mb_solarpanel_4_current: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.4.current
    :field tlm_mb_solarpanel_4_volt_pos: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.4.volt_pos
    :field tlm_mb_solarpanel_4_volt_neg: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.4.volt_neg
    :field tlm_mb_solarpanel_5_current: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.5.current
    :field tlm_mb_solarpanel_5_volt_pos: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.5.volt_pos
    :field tlm_mb_solarpanel_5_volt_neg: ax25_frame.payload.ax25_info.cdm_payload.solarpanel.5.volt_neg
    :field tlm_mb_slot_0_voltage: ax25_frame.payload.ax25_info.cdm_payload.slot.0.voltage
    :field tlm_mb_slot_0_current: ax25_frame.payload.ax25_info.cdm_payload.slot.0.current
    :field tlm_mb_slot_1_voltage: ax25_frame.payload.ax25_info.cdm_payload.slot.1.voltage
    :field tlm_mb_slot_1_current: ax25_frame.payload.ax25_info.cdm_payload.slot.1.current
    :field tlm_mb_slot_2_voltage: ax25_frame.payload.ax25_info.cdm_payload.slot.2.voltage
    :field tlm_mb_slot_2_current: ax25_frame.payload.ax25_info.cdm_payload.slot.2.current
    :field tlm_mb_slot_3_voltage: ax25_frame.payload.ax25_info.cdm_payload.slot.3.voltage
    :field tlm_mb_slot_3_current: ax25_frame.payload.ax25_info.cdm_payload.slot.3.current
    :field tlm_mb_slot_4_voltage: ax25_frame.payload.ax25_info.cdm_payload.slot.4.voltage
    :field tlm_mb_slot_4_current: ax25_frame.payload.ax25_info.cdm_payload.slot.4.current
    :field tlm_mb_slot_5_voltage: ax25_frame.payload.ax25_info.cdm_payload.slot.5.voltage
    :field tlm_mb_slot_5_current: ax25_frame.payload.ax25_info.cdm_payload.slot.5.current
    :field tlm_mb_slot_6_voltage: ax25_frame.payload.ax25_info.cdm_payload.slot.6.voltage
    :field tlm_mb_slot_6_current: ax25_frame.payload.ax25_info.cdm_payload.slot.6.current
    :field tlm_mb_slot_7_voltage: ax25_frame.payload.ax25_info.cdm_payload.slot.7.voltage
    :field tlm_mb_slot_7_current: ax25_frame.payload.ax25_info.cdm_payload.slot.7.current
    :field tlm_mb_slot_8_voltage: ax25_frame.payload.ax25_info.cdm_payload.slot.8.voltage
    :field tlm_mb_slot_8_current: ax25_frame.payload.ax25_info.cdm_payload.slot.8.current
    :field tlm_mb_solartemp_0_temp_0: ax25_frame.payload.ax25_info.cdm_payload.solar_temp.0.temp.0
    :field tlm_mb_solartemp_0_temp_1: ax25_frame.payload.ax25_info.cdm_payload.solar_temp.0.temp.1
    :field tlm_mb_solartemp_0_temp_2: ax25_frame.payload.ax25_info.cdm_payload.solar_temp.0.temp.2
    :field tlm_mb_solartemp_0_temp_3: ax25_frame.payload.ax25_info.cdm_payload.solar_temp.0.temp.3
    :field tlm_mb_solartemp_1_temp_0: ax25_frame.payload.ax25_info.cdm_payload.solar_temp.1.temp.0
    :field tlm_mb_solartemp_1_temp_1: ax25_frame.payload.ax25_info.cdm_payload.solar_temp.1.temp.1
    :field tlm_mb_solartemp_1_temp_2: ax25_frame.payload.ax25_info.cdm_payload.solar_temp.1.temp.2
    :field tlm_mb_solartemp_1_temp_3: ax25_frame.payload.ax25_info.cdm_payload.solar_temp.1.temp.3
    :field tlm_mb_oc_cnt_solar: ax25_frame.payload.ax25_info.cdm_payload.solar_bus_oc_cnt
    :field tlm_mb_oc_cnt_batt_0: ax25_frame.payload.ax25_info.cdm_payload.batt_pack_oc_cnt.0
    :field tlm_mb_oc_cnt_batt_1: ax25_frame.payload.ax25_info.cdm_payload.batt_pack_oc_cnt.1
    :field tlm_mb_powerswitch_trx: ax25_frame.payload.ax25_info.cdm_payload.power_switch_trx
    :field tlm_mb_powerswitch_0: ax25_frame.payload.ax25_info.cdm_payload.power_switch_state.raw.0
    :field tlm_mb_powerswitch_1: ax25_frame.payload.ax25_info.cdm_payload.power_switch_state.raw.1
    :field tlm_mb_powerswitch_2: ax25_frame.payload.ax25_info.cdm_payload.power_switch_state.raw.2
    :field tlm_trx_cmn_boardid: ax25_frame.payload.ax25_info.cdm_payload.common_trx.board_id_cdm
    :field tlm_trx_cmn_reset_cnt: ax25_frame.payload.ax25_info.cdm_payload.common_trx.board_rst_total_cnt
    :field tlm_trx_cmn_reset_iwdg_cnt: ax25_frame.payload.ax25_info.cdm_payload.common_trx.board_rst_iwdg_cnt
    :field tlm_trx_cmn_reset_iwdg_timestamp: ax25_frame.payload.ax25_info.cdm_payload.common_trx.board_rst_iwdg_timestamp
    :field tlm_trx_cmn_mcu_csr: ax25_frame.payload.ax25_info.cdm_payload.common_trx.mcu_rcc_csr
    :field tlm_trx_cmn_mcu_uptime: ax25_frame.payload.ax25_info.cdm_payload.common_trx.mcu_uptime
    :field tlm_trx_cmn_mcu_temp: ax25_frame.payload.ax25_info.cdm_payload.common_trx.mcu_temp
    :field tlm_trx_cmn_rtc_unixtime: ax25_frame.payload.ax25_info.cdm_payload.common_trx.rtc_unixtime
    :field tlm_trx_cmn_rtc_vbat: ax25_frame.payload.ax25_info.cdm_payload.common_trx.rtc_bat
    :field tlm_trx_boardid: ax25_frame.payload.ax25_info.cdm_payload.board_id_modem
    :field tlm_trx_vbus: ax25_frame.payload.ax25_info.cdm_payload.board_vbus
    :field tlm_trx_startup_previous: ax25_frame.payload.ax25_info.cdm_payload.startup_unixtime_previous
    :field tlm_trx_startup_current: ax25_frame.payload.ax25_info.cdm_payload.startup_unixtime_current
    :field tlm_trx_ds600_temp: ax25_frame.payload.ax25_info.cdm_payload.ds600_temp
    :field tlm_trx_tmp75_temp: ax25_frame.payload.ax25_info.cdm_payload.tmp75_temp
    :field tlm_trx_pamp_current_tx: ax25_frame.payload.ax25_info.cdm_payload.ina226_temp_current_tx
    :field tlm_trx_pamp_voltage_tx: ax25_frame.payload.ax25_info.cdm_payload.ina226_temp_voltage_tx
    :field tlm_trx_ax5043_enabled: ax25_frame.payload.ax25_info.cdm_payload.tps2042_ch1_enabled
    :field tlm_trx_ax5043_oc: ax25_frame.payload.ax25_info.cdm_payload.tps2042_ch1_oc
    :field tlm_trx_lna_enabled: ax25_frame.payload.ax25_info.cdm_payload.tps2042_ch2_enabled
    :field tlm_trx_lna_oc: ax25_frame.payload.ax25_info.cdm_payload.tps2042_ch2_oc
    :field tlm_trx_pamp_oc: ax25_frame.payload.ax25_info.cdm_payload.tps2032_oc
    :field tlm_trx_pamp_enabled: ax25_frame.payload.ax25_info.cdm_payload.tps61078_enabled
    :field tlm_trx_modem_state: ax25_frame.payload.ax25_info.cdm_payload.modem_state
    :field tlm_trx_modem_pwr_fwd: ax25_frame.payload.ax25_info.cdm_payload.modem_pwr_fwd
    :field tlm_trx_modem_pwr_rev: ax25_frame.payload.ax25_info.cdm_payload.modem_pwr_rev
    :field tlm_trx_modem_rx_freq: ax25_frame.payload.ax25_info.cdm_payload.modem_rx_freq
    :field tlm_trx_modem_rx_datarate: ax25_frame.payload.ax25_info.cdm_payload.modem_rx_datarate
    :field tlm_trx_modem_rx_mode: ax25_frame.payload.ax25_info.cdm_payload.modem_rx_mode
    :field tlm_trx_modem_rx_period_on: ax25_frame.payload.ax25_info.cdm_payload.modem_rx_period_on
    :field tlm_trx_modem_rx_period_off: ax25_frame.payload.ax25_info.cdm_payload.modem_rx_period_off
    :field tlm_trx_modem_rx_cnt_all: ax25_frame.payload.ax25_info.cdm_payload.modem_rx_cnt_all
    :field tlm_trx_modem_rx_cnt_valid: ax25_frame.payload.ax25_info.cdm_payload.modem_rx_cnt_valid
    :field tlm_trx_modem_rx_seqnum: ax25_frame.payload.ax25_info.cdm_payload.modem_rx_seqnum
    :field tlm_trx_modem_tx_freq: ax25_frame.payload.ax25_info.cdm_payload.modem_tx_freq
    :field tlm_trx_modem_tx_datarate: ax25_frame.payload.ax25_info.cdm_payload.modem_tx_datarate
    :field tlm_trx_modem_tx_pwr: ax25_frame.payload.ax25_info.cdm_payload.modem_tx_pwr
    :field tlm_trx_modem_tx_cnt_all: ax25_frame.payload.ax25_info.cdm_payload.modem_tx_cnt_all
    :field tlm_trx_modem_cnt_digipeater_ax25: ax25_frame.payload.ax25_info.cdm_payload.modem_cnt_digipeater_ax25
    :field tlm_trx_modem_cnt_digipeater_greencube_rx: ax25_frame.payload.ax25_info.cdm_payload.modem_cnt_digipeater_greencube_rx
    :field tlm_trx_modem_cnt_digipeater_greencube_tx: ax25_frame.payload.ax25_info.cdm_payload.modem_cnt_digipeater_greencube_tx
    :field tlm_trx_message: ax25_frame.payload.ax25_info.cdm_payload.text_msg
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Cubebel2.Ax25Frame(self._io, self, self._root)

    class MbSolarpanelTelemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.current = self._io.read_u2le()
            self.volt_pos = self._io.read_u2le()
            self.volt_neg = self._io.read_u2le()


    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Cubebel2.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 3:
                self.payload = Cubebel2.UiFrame(self._io, self, self._root)


    class TrxBeacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_id = self._io.read_u1()
            self.beacon_uptime = self._io.read_u4le()
            self.beacon_vbus = self._io.read_u2le()
            self.beacon_reset_total_cnt = self._io.read_u1()
            self.beacon_reset_iwdg_cnt = self._io.read_u1()
            self.beacon_reset_iwdg_time = self._io.read_u4le()
            self.beacon_pamp_temp = self._io.read_u2le()
            self.beacon_rx_settings = self._io.read_u1()
            self.beacon_rx_period_on = self._io.read_u2le()
            self.beacon_rx_seqnum = self._io.read_u2le()
            self.beacon_rx_cnt_total = self._io.read_u1()
            self.beacon_rx_cnt_valid = self._io.read_u1()
            self.beacon_tx_settings = self._io.read_u1()
            self.beacon_tx_pwr = self._io.read_s1()
            self.beacon_tx_cnt = self._io.read_u1()


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Cubebel2.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Cubebel2.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Cubebel2.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Cubebel2.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class MbSlotTelemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.voltage = self._io.read_u2le()
            self.current = self._io.read_u2le()
            self.oc_cnt = self._io.read_u1()


    class MbSolarpanelTempTelemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.temp = [None] * (4)
            for i in range(4):
                self.temp[i] = self._io.read_u2le()



    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Cubebel2.Frame(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"EU11S ") or (self.callsign == u"EU1XX ")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.trx_beacon = Cubebel2.TrxBeacon(self._io, self, self._root)
            self.cdm_datalen = self._io.read_u1()
            self.cdm_header = Cubebel2.CdmHeader(self._io, self, self._root)
            _on = self.cdm_header.cdm_id
            if _on == 33272:
                self.cdm_payload = Cubebel2.CdmConfigTrxAns(self._io, self, self._root)
            elif _on == 36770:
                self.cdm_payload = Cubebel2.CdmTrxRfreplyAns(self._io, self, self._root)
            elif _on == 33372:
                self.cdm_payload = Cubebel2.CdmConfigTrxAns(self._io, self, self._root)
            elif _on == 33179:
                self.cdm_payload = Cubebel2.CdmConfigMotherboardAns(self._io, self, self._root)
            elif _on == 33169:
                self.cdm_payload = Cubebel2.CdmTelemetryGetAnsMotherboard(self._io, self, self._root)
            elif _on == 33369:
                self.cdm_payload = Cubebel2.CdmConfigMotherboardAns(self._io, self, self._root)
            elif _on == 33172:
                self.cdm_payload = Cubebel2.CdmTelemetryGetAnsTrx(self._io, self, self._root)
            elif _on == 32770:
                self.cdm_payload = Cubebel2.CdmVersionGetAns(self._io, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class CdmTrxRfreplyAns(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax5043_timer = self._io.read_u4le()
            self.ax5043_track_rf_freq = self._io.read_u4le()
            self.ax5043_datarate = self._io.read_u4le()
            self.ax5043_track_freq = self._io.read_s2le()
            self.ax5043_rssi = self._io.read_u1()
            self.ax5043_agc = self._io.read_u1()
            self.ax5043_background_noise = self._io.read_u1()
            self.command_hash = [None] * (28)
            for i in range(28):
                self.command_hash[i] = self._io.read_u1()

            self.command_key_idx = self._io.read_u4le()
            self.command_seq_enabled = self._io.read_u1()
            self.command_seq_valid = self._io.read_u1()
            self.command_seq_num = self._io.read_u2le()


    class MbBattTelemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.element = [None] * (2)
            for i in range(2):
                self.element[i] = Cubebel2.MbBattElementTelemetry(self._io, self, self._root)

            self.voltage = self._io.read_u2le()


    class MbBattElementTelemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.voltage = self._io.read_u2le()
            self.current = self._io.read_u2le()
            self.temp = self._io.read_u2le()


    class CdmTelemetryGetAnsTrx(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.common_trx = Cubebel2.CdmTelemetryGetAnsCommonTrx(self._io, self, self._root)
            self.board_id_modem = self._io.read_u1()
            self.board_vbus = self._io.read_f4le()
            self.startup_unixtime_previous = self._io.read_u4le()
            self.startup_unixtime_current = self._io.read_u4le()
            self.ds600_inited = self._io.read_u1()
            self.ds600_enabled = self._io.read_u1()
            self.ds600_temp = self._io.read_f4le()
            self.tmp75_inited = self._io.read_u1()
            self.tmp75_temp = self._io.read_f4le()
            self.ina226_pamp_inited = self._io.read_u1()
            self.ina226_pamp_current = self._io.read_f4le()
            self.ina226_temp_current_tx = self._io.read_f4le()
            self.ina226_pamp_voltage = self._io.read_f4le()
            self.ina226_temp_voltage_tx = self._io.read_f4le()
            self.tps2042_inited = self._io.read_u1()
            self.tps2042_ch1_enabled = self._io.read_u1()
            self.tps2042_ch1_oc = self._io.read_u1()
            self.tps2042_ch2_enabled = self._io.read_u1()
            self.tps2042_ch2_oc = self._io.read_u1()
            self.tps2032_inited = self._io.read_u1()
            self.tps2032_oc = self._io.read_u1()
            self.tps61078_inited = self._io.read_u1()
            self.tps61078_enabled = self._io.read_u1()
            self.modem_inited = self._io.read_u1()
            self.modem_state = self._io.read_u1()
            self.modem_pwr_fwd = self._io.read_f4le()
            self.modem_pwr_rev = self._io.read_f4le()
            self.modem_rx_freq = self._io.read_u4le()
            self.modem_rx_datarate = self._io.read_u4le()
            self.modem_rx_mode = self._io.read_u1()
            self.modem_rx_period_on = self._io.read_u4le()
            self.modem_rx_period_off = self._io.read_u4le()
            self.modem_rx_cnt_all = self._io.read_u4le()
            self.modem_rx_cnt_valid = self._io.read_u4le()
            self.modem_rx_seqnum = self._io.read_u4le()
            self.modem_tx_freq = self._io.read_u4le()
            self.modem_tx_datarate = self._io.read_u4le()
            self.modem_tx_pwr = self._io.read_s1()
            self.modem_tx_cnt_all = self._io.read_u4le()
            self.modem_cnt_digipeater_ax25 = self._io.read_u4le()
            self.modem_cnt_digipeater_greencube_rx = self._io.read_u4le()
            self.modem_cnt_digipeater_greencube_tx = self._io.read_u4le()
            self.text_msg = (self._io.read_bytes_full()).decode(u"ASCII")


    class CdmTelemetryGetAnsMotherboard(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.time = self._io.read_u2le()
            self.mcusr = self._io.read_u1()
            self.rst_cnt_total = self._io.read_u1()
            self.rst_cnt_iwdg = self._io.read_u1()
            self.adc_status = self._io.read_u1()
            self.adc_temp_1 = self._io.read_u2le()
            self.adc_temp_2 = self._io.read_u2le()
            self.ant_1_v = self._io.read_u2le()
            self.ant_2_v = self._io.read_u2le()
            self.solar_common_v = self._io.read_u2le()
            self.sat_bus_v = self._io.read_u2le()
            self.sat_bus_c = self._io.read_u2le()
            self.uc_v = self._io.read_u2le()
            self.uc_c = self._io.read_u2le()
            self.battery_back = [None] * (2)
            for i in range(2):
                self.battery_back[i] = Cubebel2.MbBattTelemetry(self._io, self, self._root)

            self.solarpanel = [None] * (6)
            for i in range(6):
                self.solarpanel[i] = Cubebel2.MbSolarpanelTelemetry(self._io, self, self._root)

            self.slot = [None] * (9)
            for i in range(9):
                self.slot[i] = Cubebel2.MbSlotTelemetry(self._io, self, self._root)

            self.solar_temp = [None] * (2)
            for i in range(2):
                self.solar_temp[i] = Cubebel2.MbSolarpanelTempTelemetry(self._io, self, self._root)

            self.solar_bus_oc_cnt = self._io.read_u1()
            self.batt_pack_oc_cnt = [None] * (2)
            for i in range(2):
                self.batt_pack_oc_cnt[i] = self._io.read_u1()

            self.power_switch_trx = self._io.read_u2le()
            self.power_switch_state = Cubebel2.MbPowerSwitchState(self._io, self, self._root)


    class CdmConfigMotherboardRecord(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.id = self._io.read_u1()
            self.value = self._io.read_u2le()


    class CdmConfigMotherboardAns(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.record_mb = []
            i = 0
            while not self._io.is_eof():
                self.record_mb.append(Cubebel2.CdmConfigMotherboardRecord(self._io, self, self._root))
                i += 1



    class MbPowerSwitchState(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.raw = [None] * (3)
            for i in range(3):
                self.raw[i] = self._io.read_u1()



    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Cubebel2.Callsign(_io__raw_callsign_ror, self, self._root)


    class CdmConfigStm32l4Record(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.id = self._io.read_u1()
            self.value = self._io.read_u4le()


    class CdmVersionGetAns(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.version_githash = (self._io.read_bytes_full()).decode(u"ASCII")


    class CdmHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cdm_id = self._io.read_u2le()
            self.cdm_addr_src = self._io.read_u1()
            self.cdm_addr_dst = self._io.read_u1()
            self.cdm_priority = self._io.read_u1()
            self.cdm_delay = self._io.read_u2le()
            self.cdm_datalen = self._io.read_u1()


    class CdmConfigTrxAns(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.record_trx = []
            i = 0
            while not self._io.is_eof():
                self.record_trx.append(Cubebel2.CdmConfigStm32l4Record(self._io, self, self._root))
                i += 1



    class CdmTelemetryGetAnsCommonTrx(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.board_id_cdm = self._io.read_u1()
            self.board_rst_total_cnt = self._io.read_u1()
            self.board_rst_iwdg_cnt = self._io.read_u1()
            self.board_rst_iwdg_timestamp = self._io.read_u4le()
            self.mcu_rcc_csr = self._io.read_u1()
            self.mcu_uptime = self._io.read_u4le()
            self.rtc_unixtime = self._io.read_s8le()
            self.rtc_bat = self._io.read_f4le()
            self.mcu_temp = self._io.read_f4le()



